# Priority Queue with binary heap

class PriorityQueue():
  DEFAULT_CAPACITY = 100

  def __init__(self, els=None):
    self._data = [ None ] * PriorityQueue.DEFAULT_CAPACITY
    self._size = 0
      
  def __len__(self):
    return self._size

  def findmin(self):
    if self._size == 0:
      raise ValueError("empty priority queue")
    return self._data[1]

  def insert(self, x):
    if self._size + 1 == len(self._data):
      # double size of the array storing the data
      self._data.extend( [ None ] * len(self._data) )
    self._size += 1
    hole = self._size
    self._data[0] = x
    # Percolate up
    while x < self._data[hole // 2]:
      self._data[hole] = self._data[hole // 2]
      hole //= 2
    self._data[hole] = x
    
  def deletemin(self):
    min_item = self.findmin()         # raises error if empty
    self._data[1] = self._data[self._size]
    self._size -= 1
    self._percolate_down(1)
    return min_item

  def _percolate_down(self, i):
    in_hole = self._data[i]
    hole = i
    child = self._smaller_child(hole, in_hole)
    while child != 0:
      self._data[hole] = self._data[child]
      hole = child
      child = self._smaller_child(hole, in_hole)
    self._data[hole] = in_hole

  # Is one child of node hole smaller than element in_hole?
  # Returns index of child, or 0 if no child is smaller
  def _smaller_child(self, hole, in_hole):
    if 2 * hole <= self._size:
      child = 2 * hole
      if child != self._size and self._data[child + 1] < self._data[child]:
        child += 1
      if self._data[child] < in_hole:
        return child
    return 0
